package org.wamblee.support.jndi;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;

/**
 * Test initial context factory used for testing software in a Java SE
 * environnment that uses JNDI to retrieve objects.
 * 
 * See {@link #bind(String, Object)} to resp. register the initial context.
 * 
 * To bind objects in the JNDI tree simply use the standard JNDI api: <code>
 *   InitialContext context = new InitialContext();
 *   MyClass myObj = ...; 
 *   context.bind("a/b", myObj); 
 * </code>
 */
public class StubInitialContextFactory implements InitialContextFactory {

	private static Context context;

	private static void initialize() {
		try {
			context = new StubInitialContext();
		} catch (NamingException e) { // can't happen.
			throw new RuntimeException(e);
		}
	}

	/**
	 * This method must be called to register this initial context factory as
	 * the default implementation for JNDI.
	 * 
	 * @throws Exception
	 */
	public static void register() {
		// sets up the InitialContextFactoryForTest as default factory.
		System.setProperty(Context.INITIAL_CONTEXT_FACTORY,
				StubInitialContextFactory.class.getName());
		if (context == null) {
			initialize();
		}
	}
	
	/**
	 * Unregisters the initial context factory 
	 */
	public static void unregister() { 
		System.setProperty(Context.INITIAL_CONTEXT_FACTORY, "");
		context = null; 
	}

	public Context getInitialContext(Hashtable<?, ?> environment)
			throws NamingException {
		return context;
	}
}
