package org.wamblee.support.persistence;

/**
 * Database provider. This database provider represents a particular type of
 * database such as its capabilities and the ability to create an instance
 * representing a running database.
 * 
 * Since the {@link DatabaseBuilder} uses a first match algorithm and the order
 * of databaseproviders is not guaranteed, it is recommended for each database
 * provider to also provide a unique capability that no other database has.
 */
public interface DatabaseProvider {

	/**
	 * Capability that all databases that run inmemory have.
	 */
	String CAPABILITY_IN_MEMORY = "INMEMORY";

	/**
	 * Capability that all databases that are external have.
	 */
	String CAPABILITY_EXTERNAL = "EXTERNAL";

	/**
	 * Determines if the database has all capabilities that are requested.
	 * @param aCapabilities Capabilities it must ahve
	 * @return True if it has all capabilities. 
	 */
	boolean supportsCapabilities(String[] aCapabilities);

	/**
	 * Gets the description for the database. 
	 * @return Description.
	 */
	DatabaseDescription getDescription();

	/**
	 * Creates a database instance that represents a running instance of that database. 
	 * @return Database. 
	 */
	Database create();
}
